# Sentiment analysis and structural breaks identification module

import jenkspy
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure
import numpy as np
import pandas as pd
import sys


from .preprocess import *
from .group import *
from .sentiment import *



def coffee_break(text: str,                  # Text column
                 time: str,                  # Time column
                 date_format: str,           # Date format: 'eur' - European, 'us' - American
                 model: str,                 # Sentiment classifier: 'vader' - general language,'finvader' - financial texts
                 skip: [ ],                  # Remove additional strings
                 preprocess: bool = False,   # Clean data from numbers and punctuation
                 time_freq: int = '',        # Aggregation period: 'Y'/'M'
                 n_breaks: int = ''):        # Number of breaks: min. 2

    date_formats = ["eur", "us"]
    frequencies = ["M", "Y"]

    if date_format in date_formats:
        pass
    else:
        sys.exit('Incorrect value for date_format parameter. Allowed: "eur", "us"')


    if time_freq in frequencies:
        pass
    else:
        sys.exit('Incorrect value for time_freq parameter. Allowed: "M", "Y"')


    if n_breaks is not None:
        if n_breaks < 2:
            sys.exit("N_breaks parameter is too low. Increase it!")

        else:
            pass

    data=pd.DataFrame()
    data['text']=text
    data['period']=time

    data=data.set_index(data['period'])
    data['text'].replace('', np.nan, inplace=True)
    data['text'].replace(' ', np.nan, inplace=True)
    data.dropna(subset=['text'], inplace=True)

    if skip is not None:
        data['text'] = data.text.str.replace('|'.join(skip), '.', regex=True).str.strip()

    if preprocess is True:
        data['text'] = np.vectorize(preprocess_sentiment)(data['text'])

    if time_freq == 'M':
        if date_format == 'eur':
            data['period']=data['period'].astype(str)
            data['period'] = pd.to_datetime(data['period'], dayfirst=True, errors = 'coerce')

        elif date_format == 'us':
            data['period']=data['period'].astype(str)
            data['period'] = pd.to_datetime(data['period'], dayfirst=False, errors = 'coerce')

        if model == 'vader':
            data['sentiment'] = np.vectorize(sentiment_vader)(data['text'])
        elif model == 'finvader':
            data['sentiment'] = np.vectorize(fin_vader)(data['text'])

        sentiment_ts = grouper_sentiment(data = data,time_freq='1M')
        sentiment_ts['period'] = pd.to_datetime(sentiment_ts['period'])
        sentiment_ts.set_index(sentiment_ts['period'], inplace = True)
        sentiment_ts.columns = ['period', 'sentiment']

        if n_breaks is None:
            ts = sentiment_ts['sentiment']
            ts = pd.DataFrame(ts)

            ts.index = pd.to_datetime(ts.index, format = '%Y-%m')
            sentiment_ts.reset_index(drop = True, inplace = True)

            sentiment_ts['period'] = pd.to_datetime(sentiment_ts['period'].astype('str'))
            sentiment_ts['period'] = sentiment_ts['period'].dt.strftime('%Y-%m')
            sentiment_ts.set_index('period', inplace = True)

            figure(figsize=(12, 8), dpi=800)
            plt.plot(ts, label='sentiment')
            plt.grid(linewidth=0.5)
            plt.xlabel('Period', size = 13, labelpad = 6)
            plt.ylabel('Sentiment', size = 13, labelpad = 6)
            plt.legend(loc = 'best', fontsize = 13)
            picture = plt.show()

        elif n_breaks is not None:
            sentiment_ts.reset_index(drop = True, inplace = True)
            sentiment_ts['period'] = pd.to_datetime(sentiment_ts['period'].astype('str'))
            sentiment_ts['period'] = sentiment_ts['period'].dt.strftime('%Y-%m')

            ts = sentiment_ts
            ts.set_index('period', inplace = True)
            ts.index = pd.to_datetime(ts.index, format='%Y-%m').to_period("m")
            ts_series = ts.squeeze()

            y = np.array(ts_series.tolist())
            breaks = jenkspy.jenks_breaks(y, n_classes=n_breaks-1)

            breaks_jkp = []

            for v in breaks:
                idx = ts_series.index[ts_series == v]
                breaks_jkp.append(idx)

            figure(figsize=(12, 8), dpi=800)
            ts_series.plot(label='sentiment')
            print_legend = True

            for i in breaks_jkp:
                if print_legend:
                    plt.axvline(i, color='red',linestyle='dashed', label='breaks')
                    print_legend = False
                else:
                    plt.axvline(i, color='red',linestyle='dashed')

            plt.grid(linewidth=0.5)
            plt.xlabel('Period', size = 13, labelpad = 6)
            plt.ylabel('Sentiment', size = 13, labelpad = 6)
            plt.legend(loc = 'best', fontsize = 13)
            picture = plt.show()


    elif time_freq == 'Y':
        if date_format == 'eur':
            data['period']=data['period'].astype(str)
            data['period'] = pd.to_datetime(data['period'], dayfirst=True,errors = 'coerce')

        elif date_format == 'us':
            data['period']=data['period'].astype(str)
            data['period'] = pd.to_datetime(data['period'], dayfirst=False,errors = 'coerce')

        if model == 'vader':
            data['sentiment'] = np.vectorize(sentiment_vader)(data['text'])
        elif model == 'finvader':
            data['sentiment'] = np.vectorize(fin_vader)(data['text'])

        sentiment_ts = grouper_sentiment(data = data,time_freq='1Y')
        sentiment_ts['period'] = pd.to_datetime(sentiment_ts['period'])
        sentiment_ts.set_index(sentiment_ts['period'], inplace = True)
        sentiment_ts.columns = ['period', 'sentiment']

        if n_breaks is None:
            ts = sentiment_ts['sentiment']
            ts = pd.DataFrame(ts)

            ts.index = pd.to_datetime(ts.index, format = '%Y').year
            sentiment_ts.reset_index(drop = True, inplace = True)

            sentiment_ts['period'] = pd.to_datetime(sentiment_ts['period'].astype('str'))
            sentiment_ts['period'] = sentiment_ts['period'].dt.strftime('%Y')

            constant = (len(sentiment_ts['period']) / 10) # TADY
            sentiment_ts.set_index('period', inplace = True)
            figure(figsize=((12 * (1 + (0.1 * constant))), 8), dpi=800)  # TADY

            plt.plot(ts, label='sentiment')
            plt.grid(linewidth=0.5)
            plt.xlabel('Period', size = 13, labelpad = 8)
            plt.xticks(ts.index, rotation = 45)
            plt.ylabel('Sentiment', size = 13, labelpad = 6)
            plt.legend(loc = 'best', fontsize = 13)
            picture = plt.show()

        elif n_breaks is not None:
            sentiment_ts.reset_index(drop = True, inplace = True)
            sentiment_ts['period'] = pd.to_datetime(sentiment_ts['period'].astype('str'))
            sentiment_ts['period'] = sentiment_ts['period'].dt.strftime('%Y')

            constant = (len(sentiment_ts['period']) / 10) # TADY
            ts = sentiment_ts

            ts.set_index('period', inplace = True)
            ts.index = pd.to_datetime(ts.index, format='%Y').year
            ts_series = ts.squeeze()

            y = np.array(ts_series.tolist())
            breaks = jenkspy.jenks_breaks(y, n_classes=n_breaks-1)

            breaks_jkp = []

            for v in breaks:
                idx = ts_series.index[ts_series == v]
                breaks_jkp.append(idx)

            figure(figsize=((12 * (1 + (0.1 * constant))), 8), dpi=800)  # TADY
            plt.plot(ts_series, label='sentiment')
            print_legend = True

            for i in breaks_jkp:
                if print_legend:
                    plt.axvline(i, color='red',linestyle='dashed', label='breaks')
                    print_legend = False
                else:
                    plt.axvline(i, color='red',linestyle='dashed')

            plt.grid(linewidth=0.5)
            plt.xlabel('Period', size = 13, labelpad = 6)
            plt.xticks(ts_series.index, rotation = 45)
            plt.ylabel('Sentiment', size = 13, labelpad = 6)
            plt.legend(loc = 'best', fontsize = 13)
            picture = plt.show()

    return sentiment_ts


# Data visualization module

from plotnine import *
from arabica import arabica_freq
from wordcloud import WordCloud
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure
import pandas as pd
import numpy as np
import sys



def cappuccino(text: str,                  # Text
               time: str,                  # Time
               date_format: str,           # Date format: 'eur' - European, 'us' - American
               stopwords: [],              # Languages for stop words
               stopwords_ext: [],          # Languages for extended stop words lists
               skip: [ ],                  # Remove additional strings
               plot: str = '',             # Chart type: 'wordcloud'/'heatmap'/'line'
               ngram: int = '',            # N-gram size, 1 = unigram, 2 = bigram, 3 = trigram
               time_freq: str = '',        # Aggregation period: 'Y'/'M', if no aggregation: 'ungroup'
               max_words: int = '',        # Max number for most frequent n-grams displayed for each period
               numbers: bool = False,      # Remove numbers
               lower_case: bool = False):  # Lowercase text before cleaning and frequency analysis

    result = arabica_freq(text = text,
                          time = time,
                          date_format = date_format,
                          time_freq = time_freq,
                          max_words = max_words,
                          stopwords = stopwords,
                          stopwords_ext = stopwords_ext,
                          skip = skip,
                          numbers = numbers,
                          lower_case = lower_case)

    if ngram == 1:
        if time_freq == 'ungroup':
            if plot == 'wordcloud':
                unigrams_df = result[['unigram','unigram_freq']]
                unigrams_df = unigrams_df.dropna()
                unigrams_df['unigram_freq'] = unigrams_df['unigram_freq'].astype(int)
                unigrams_dict = dict(unigrams_df.values)
                fig = figure()
                wordcloud_unigram = WordCloud(background_color="white",
                                              colormap='tab20b',
                                              max_font_size=250,
                                              width=2000,
                                              height=1200,
                                              margin=10).generate_from_frequencies(unigrams_dict)
                plt.figure(dpi=1200)
                plt.imshow(wordcloud_unigram,interpolation="bilinear")
                fig.set_size_inches(10, 6, forward=True)
                plt.axis('off')
                picture = plt.show()

            else:
                sys.exit('Incorrect value for plot parameter. Allowed: "wordcloud"')


        elif time_freq == 'M':
            period = result['period']
            subset = result['unigram']
            subset = subset.dropna()
            subset_expand = subset.str.split(pat=",", expand=True)
            subset_expand.columns = ['unigram' + str(i + 1) for i in range(len(subset_expand.columns))]
            subset_expand["id"] = subset_expand.index
            test = subset_expand.merge(period, left_index = True, right_index = True)
            reshaped = pd.wide_to_long(test, ['unigram'], i="id", j="period")
            period_re = reshaped['period']
            period_re.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped['unigram']
            reshaped_unigram.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped_unigram.str.split(pat=": ", expand=True)
            reshaped_unigram = reshaped_unigram.rename(columns={0: 'unigram', 1: 'frequency'})
            reshaped_unigram = reshaped_unigram.dropna()
            reshaped_unigram['frequency'] = reshaped_unigram['frequency'].astype(int)
            reshaped_unigram = reshaped_unigram.merge(period_re, left_index = True, right_index = True)

            if plot == 'heatmap':
                reshaped_unigram = reshaped['unigram']
                reshaped_unigram.reset_index(drop=True, inplace=True)
                reshaped_unigram = reshaped_unigram.str.split(pat=": ", expand=True)
                reshaped_unigram = reshaped_unigram.rename(columns={0: 'unigram', 1: 'frequency'})
                reshaped_unigram = reshaped_unigram.dropna()
                reshaped_unigram['frequency'] = reshaped_unigram['frequency'].astype(int)
                reshaped_unigram = reshaped_unigram.merge(period_re, left_index = True, right_index = True)
                reshaped_unigram_max = reshaped_unigram.groupby(['unigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram = reshaped_unigram.merge(reshaped_unigram_max, on = ['unigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram['label'] = np.where(reshaped_unigram['_merge']=='both', reshaped_unigram['unigram'], '')
                reshaped_unigram = reshaped_unigram[['unigram', 'frequency', 'period','label']]

                fig = (ggplot(reshaped_unigram, aes('period', 'unigram', fill='frequency'))
                       + geom_tile(aes(width=1, height=1))
                       + geom_text(aes(label='frequency'), size=10)
                       + theme_minimal()
                       + scale_fill_gradient(name = "frequency",
                                             low = "#f0f4ed",
                                             high = "#496160")
                       + theme(axis_title_x=element_text(colour = "black"),
                               axis_title_y=element_text(colour="black"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust= 1),
                               legend_text = element_text(size = 7),
                               legend_title = element_text(color = "black", vjust= -1.3, size = 10),
                               figure_size= (3 + ((len(period) /4)), (3 + (max_words * 1.5))),
                               panel_grid_major = element_line(color='black',size = 0.1),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       ).draw(show=False, return_ggplot=True)

                plt.gcf().set_dpi(800)
                picture = plt.show()


            elif plot == 'line':
                reshaped_unigram_wide = pd.pivot(reshaped_unigram, index = 'unigram', columns = 'period', values = 'frequency')
                reshaped_unigram_wide = reshaped_unigram_wide.fillna(0)
                year_list=list(reshaped_unigram_wide.columns)
                reshaped_unigram_wide = pd.melt(reshaped_unigram_wide, value_vars=year_list,value_name='frequency', ignore_index=False)
                reshaped_unigram_wide = reshaped_unigram_wide.reset_index()
                reshaped_unigram_max = reshaped_unigram_wide.groupby(['unigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram_wide = reshaped_unigram_wide.merge(reshaped_unigram_max, on = ['unigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram_wide['label'] = np.where(reshaped_unigram_wide['_merge']=='both', reshaped_unigram_wide['unigram'], '')
                reshaped_unigram_wide = reshaped_unigram_wide[['unigram', 'frequency', 'period','label']]

                fig = (ggplot(reshaped_unigram_wide)
                       + geom_point(aes(x = 'period', y = 'frequency',color = 'unigram'))
                       + geom_line(aes(x = 'period', y = 'frequency', color = 'unigram',group='unigram'))
                       + geom_label(aes(x = 'period', y = 'frequency', label = 'label',size = 1), nudge_y=2)
                       + scale_size_continuous(guide = None)
                       + guides(color=guide_legend(nrow=40))
                       + theme_minimal()
                       + theme(axis_title_x=element_text(colour = "black",size = 7),
                               axis_title_y=element_text(colour="black", size = 7),
                               axis_title=element_text(face="bold"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust=1, size = 7),
                               legend_text = element_text(size=7),
                               legend_title = element_text(size=10),
                               legend_key_height = 0.5,
                               legend_key_width = 0.5,
                               figure_size=(10, 5),
                               panel_grid_major = element_blank(),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       ).draw(show=False, return_ggplot=True)


                plt.gcf().set_dpi(800)
                picture = plt.show()

            else:
                sys.exit('Incorrect value for plot parameter. Allowed: "heatmap", "line"')


        elif time_freq == 'Y':
            period = result['period']
            subset = result['unigram']
            subset = subset.dropna()
            subset_expand = subset.str.split(pat=",", expand=True)
            subset_expand.columns = ['unigram' + str(i + 1) for i in range(len(subset_expand.columns))]
            subset_expand["id"] = subset_expand.index
            test = subset_expand.merge(period, left_index = True, right_index = True)
            reshaped = pd.wide_to_long(test, ['unigram'], i="id", j="period")
            period_re = reshaped['period']
            period_re.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped['unigram']
            reshaped_unigram.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped_unigram.str.split(pat=": ", expand=True)
            reshaped_unigram = reshaped_unigram.rename(columns={0: 'unigram', 1: 'frequency'})
            reshaped_unigram = reshaped_unigram.dropna()
            reshaped_unigram['frequency'] = reshaped_unigram['frequency'].astype(int)
            reshaped_unigram = reshaped_unigram.merge(period_re, left_index = True, right_index = True)

            if plot == 'heatmap':
                reshaped_unigram = reshaped['unigram']
                reshaped_unigram.reset_index(drop=True, inplace=True)
                reshaped_unigram = reshaped_unigram.str.split(pat=": ", expand=True)
                reshaped_unigram = reshaped_unigram.rename(columns={0: 'unigram', 1: 'frequency'})
                reshaped_unigram = reshaped_unigram.dropna()
                reshaped_unigram['frequency'] = reshaped_unigram['frequency'].astype(int)
                reshaped_unigram = reshaped_unigram.merge(period_re, left_index = True, right_index = True)
                reshaped_unigram_max = reshaped_unigram.groupby(['unigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram = reshaped_unigram.merge(reshaped_unigram_max, on = ['unigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram['label'] = np.where(reshaped_unigram['_merge']=='both', reshaped_unigram['unigram'], '')
                reshaped_unigram = reshaped_unigram[['unigram', 'frequency', 'period','label']]

                fig = (ggplot(reshaped_unigram, aes('period', 'unigram', fill='frequency'))
                       + geom_tile(aes(width=0.95, height=0.95))
                       + geom_text(aes(label='frequency'), size=7)
                       + theme_minimal()
                       + theme(axis_title_x=element_text(colour = "black"),
                               axis_title_y=element_text(colour="black"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust= 1),
                               legend_text = element_text(size = 7),
                               legend_title = element_text(color = "black", size = 10),
                               figure_size= (3 + ((len(period) /4)), (2 + (max_words * 0.5))),
                               panel_grid_major = element_line(color='black',size = 0.1),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       + scale_fill_gradient(name = "frequency",
                                             low = "#f0f4ed",
                                             high = "#496160")
                       ).draw(show=False, return_ggplot=True)

                plt.gcf().set_dpi(800)
                picture = plt.show()

            elif plot == 'line':
                reshaped_unigram_wide = pd.pivot(reshaped_unigram, index = 'unigram', columns = 'period', values = 'frequency')
                reshaped_unigram_wide = reshaped_unigram_wide.fillna(0)
                year_list=list(reshaped_unigram_wide.columns)
                reshaped_unigram_wide = pd.melt(reshaped_unigram_wide, value_vars=year_list,value_name='frequency', ignore_index=False)
                reshaped_unigram_wide = reshaped_unigram_wide.reset_index()
                reshaped_unigram_max = reshaped_unigram_wide.groupby(['unigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram_wide = reshaped_unigram_wide.merge(reshaped_unigram_max, on = ['unigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram_wide['label'] = np.where(reshaped_unigram_wide['_merge']=='both', reshaped_unigram_wide['unigram'], '')
                reshaped_unigram_wide = reshaped_unigram_wide[['unigram', 'frequency', 'period','label']]

                fig = (ggplot(reshaped_unigram_wide)
                       + geom_point(aes(x = 'period', y = 'frequency',color = 'unigram'))
                       + geom_line(aes(x = 'period', y = 'frequency', color = 'unigram',group='unigram'))
                       + geom_label(aes(x = 'period', y = 'frequency', label = 'label',size = 0.2), nudge_y=0.3)
                       + scale_size_continuous(guide = None)
                       + guides(color=guide_legend(nrow=40))
                       + theme_minimal()
                       + theme(axis_title_x=element_text(colour = "black",size = 7),
                               axis_title_y=element_text(colour="black", size = 7),
                               axis_title=element_text(face="bold"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust=1, size = 7),
                               legend_text = element_text(size=7),
                               legend_title = element_text(size=10),
                               legend_key_height = 0.5,
                               legend_key_width = 0.5,
                               figure_size=(5 + max_words, (1 + (max_words * 1.0))),
                               panel_grid_major = element_blank(),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       ).draw(show=False, return_ggplot=True)


                plt.gcf().set_dpi(800)
                picture = plt.show()

            else:
                sys.exit('Incorrect value for plot parameter. Allowed: "heatmap", "line"')

        else:
            sys.exit('Incorrect value for time_freq parameter. Allowed: "ungroup", "Y", "M"')

    elif ngram == 2:
        if time_freq == 'ungroup':
            if plot == 'wordcloud':
                bigram_df = result[['bigram','bigram_freq']]
                bigram_df = bigram_df.dropna()
                bigram_df['bigram'] = bigram_df['bigram'].str.replace(",", " ")
                bigram_df['bigram_freq'] = bigram_df['bigram_freq'].astype(int)
                bigrams_dict = dict(bigram_df.values)
                fig = figure()
                wordcloud_bigram = WordCloud(background_color="white",
                                             colormap='tab20b',
                                             width=2000,
                                             height=1200,
                                             margin=10
                                             ).generate_from_frequencies(bigrams_dict)
                plt.figure(dpi=1200)
                plt.imshow(wordcloud_bigram,interpolation="bilinear")
                fig.set_size_inches(10, 6, forward=True)
                plt.axis('off')
                picture = plt.show()

            else:
                sys.exit('Incorrect value for plot parameter. Allowed: "wordcloud"')

        elif time_freq == 'M':
            period = result['period']
            subset = result['bigram']
            subset = subset.dropna()
            subset_expand = subset.str.split(pat=",", expand=True)

            cols = (' '.join(w) for w in zip(subset_expand.columns[::2].astype(str), subset_expand.columns[1::2].astype(str)))
            subset_expand_adj = pd.DataFrame(subset_expand.iloc[:, ::2].astype(str).values +
                                             ' ' + subset_expand.iloc[:, 1::2].astype(str).values +
                                             ' ', index=subset_expand.index, columns=cols)

            subset_expand_adj.columns = ['bigram' + str(i + 1) for i in range(len(subset_expand_adj.columns))]
            subset_expand_adj["id"] = subset_expand_adj.index
            test = subset_expand_adj.merge(period, left_index = True, right_index = True)
            reshaped = pd.wide_to_long(test, ['bigram'], i="id", j="period")
            period_re = reshaped['period']
            period_re.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped['bigram']
            reshaped_unigram.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped_unigram.str.split(pat=": ", expand=True)
            reshaped_unigram = reshaped_unigram.rename(columns={0: 'bigram', 1: 'frequency'})
            reshaped_unigram = reshaped_unigram.dropna()
            reshaped_unigram['frequency'] = reshaped_unigram['frequency'].astype(int)
            reshaped_unigram = reshaped_unigram.merge(period_re, left_index = True, right_index = True)

            if plot == 'heatmap':
                reshaped_unigram_max = reshaped_unigram.groupby(['bigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram = reshaped_unigram.merge(reshaped_unigram_max, on = ['bigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram['label'] = np.where(reshaped_unigram['_merge']=='both', reshaped_unigram['bigram'], '')
                reshaped_unigram = reshaped_unigram[['bigram', 'frequency', 'period','label']]

                fig = (ggplot(reshaped_unigram, aes('period', 'bigram', fill='frequency'))
                       + geom_tile(aes(width=.95, height=.95))
                       + geom_text(aes(label='frequency'), size=10)
                       + theme_minimal()
                       + scale_fill_gradient(name = "frequency",
                                             low = "#f0f4ed",
                                             high = "#496160")
                       + theme(axis_title_x=element_text(colour = "black"),
                               axis_title_y=element_text(colour="black"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust= 1),
                               legend_text = element_text(size = 5,vjust = 0.2),
                               legend_title = element_text(color = "black", vjust = 0.8, size = 10),
                               figure_size= (3 + ((len(period) /4)), (2 + (max_words * 2.2))),
                               panel_grid_major = element_line(color='black',size = 0.1),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       ).draw(show=False, return_ggplot=True)


                plt.gcf().set_dpi(800)
                picture = plt.show()


            elif plot == 'line':
                reshaped_unigram_wide = pd.pivot(reshaped_unigram, index = 'bigram', columns = 'period', values = 'frequency')
                reshaped_unigram_wide = reshaped_unigram_wide.fillna(0)
                year_list=list(reshaped_unigram_wide.columns)
                reshaped_unigram_wide = pd.melt(reshaped_unigram_wide, value_vars=year_list,value_name='frequency', ignore_index=False)
                reshaped_unigram_wide = reshaped_unigram_wide.reset_index()
                reshaped_unigram_max = reshaped_unigram_wide.groupby(['bigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram_wide = reshaped_unigram_wide.merge(reshaped_unigram_max, on = ['bigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram_wide['label'] = np.where(reshaped_unigram_wide['_merge']=='both', reshaped_unigram_wide['bigram'], '')
                reshaped_unigram_wide = reshaped_unigram_wide[['bigram', 'frequency', 'period','label']]


                fig = (ggplot(reshaped_unigram_wide)
                       + geom_point(aes(x = 'period', y = 'frequency',color = 'bigram'))
                       + geom_line(aes(x = 'period', y = 'frequency', color = 'bigram',group='bigram'))
                       + geom_label(aes(x = 'period', y = 'frequency', label = 'label',size = 0.2), nudge_y=0.3)
                       + scale_size_continuous(guide = None)
                       + guides(color=guide_legend(nrow=40))
                       + theme_minimal()
                       + theme(axis_title_x=element_text(colour = "black",size = 7),
                               axis_title_y=element_text(colour="black", size = 7),
                               axis_title=element_text(face="bold"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust=1, size = 7),
                               legend_text = element_text(size=7),
                               legend_title = element_text(size=10),
                               legend_key_height = 0.5,
                               legend_key_width = 0.5,
                               figure_size=(10, 5),
                               panel_grid_major = element_blank(),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       ).draw(show=False, return_ggplot=True)

                plt.gcf().set_dpi(800)
                picture = plt.show()

            else:
                sys.exit('Incorrect value for plot parameter. Allowed: "heatmap", "line"')


        elif time_freq == 'Y':
            period = result['period']
            subset = result['bigram']
            subset = subset.dropna()
            subset_expand = subset.str.split(pat=",", expand=True)

            cols = (' '.join(w) for w in zip(subset_expand.columns[::2].astype(str), subset_expand.columns[1::2].astype(str)))
            subset_expand_adj = pd.DataFrame(subset_expand.iloc[:, ::2].astype(str).values +
                                             ' ' + subset_expand.iloc[:, 1::2].astype(str).values +
                                             ' ', index=subset_expand.index, columns=cols)

            subset_expand_adj.columns = ['bigram' + str(i + 1) for i in range(len(subset_expand_adj.columns))]
            subset_expand_adj["id"] = subset_expand_adj.index
            test = subset_expand_adj.merge(period, left_index = True, right_index = True)
            reshaped = pd.wide_to_long(test, ['bigram'], i="id", j="period")
            period_re = reshaped['period']
            period_re.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped['bigram']
            reshaped_unigram.reset_index(drop=True, inplace=True)
            reshaped_unigram = reshaped_unigram.str.split(pat=": ", expand=True)
            reshaped_unigram = reshaped_unigram.rename(columns={0: 'bigram', 1: 'frequency'})
            reshaped_unigram = reshaped_unigram.dropna()
            reshaped_unigram['frequency'] = reshaped_unigram['frequency'].astype(int)
            reshaped_unigram = reshaped_unigram.merge(period_re, left_index = True, right_index = True)

            if plot == 'heatmap':
                reshaped_unigram_max = reshaped_unigram.groupby(['bigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram = reshaped_unigram.merge(reshaped_unigram_max, on = ['bigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram['label'] = np.where(reshaped_unigram['_merge']=='both', reshaped_unigram['bigram'], '')
                reshaped_unigram = reshaped_unigram[['bigram', 'frequency', 'period','label']]

                fig = (ggplot(reshaped_unigram, aes('period', 'bigram', fill='frequency'))
                       + geom_tile(aes(width=.95, height=.95))
                       + geom_text(aes(label='frequency'), size=10)
                       + theme_minimal()
                       + theme(axis_title_x=element_text(colour = "black"),
                               axis_title_y=element_text(colour="black"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust= 1),
                               legend_text = element_text(size = 5,vjust = 0.2),
                               legend_title = element_text(color = "black", vjust = 0.8, size = 10),
                               figure_size= (3 + ((len(period) /4)), (2 + (max_words * 1.7))),
                               panel_grid_major = element_line(color='black',size = 0.1),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       + scale_fill_gradient(name = "frequency",
                                             low = "#f0f4ed",
                                             high = "#496160")
                       ).draw(show=False, return_ggplot=True)

                plt.gcf().set_dpi(800)
                picture = plt.show()


            elif plot == 'line':
                reshaped_unigram_wide = pd.pivot(reshaped_unigram, index = 'bigram', columns = 'period', values = 'frequency')
                reshaped_unigram_wide = reshaped_unigram_wide.fillna(0)
                year_list=list(reshaped_unigram_wide.columns)
                reshaped_unigram_wide = pd.melt(reshaped_unigram_wide, value_vars=year_list,value_name='frequency', ignore_index=False)
                reshaped_unigram_wide = reshaped_unigram_wide.reset_index()
                reshaped_unigram_max = reshaped_unigram_wide.groupby(['bigram'], sort=False)['frequency'].max()
                reshaped_unigram_max = reshaped_unigram_max.reset_index()
                reshaped_unigram_wide = reshaped_unigram_wide.merge(reshaped_unigram_max, on = ['bigram','frequency'], how = 'left', indicator=True)
                reshaped_unigram_wide['label'] = np.where(reshaped_unigram_wide['_merge']=='both', reshaped_unigram_wide['bigram'], '')
                reshaped_unigram_wide = reshaped_unigram_wide[['bigram', 'frequency', 'period','label']]


                fig = (ggplot(reshaped_unigram_wide)
                       + geom_point(aes(x = 'period', y = 'frequency',color = 'bigram'))
                       + geom_line(aes(x = 'period', y = 'frequency', color = 'bigram',group='bigram'))
                       + geom_label(aes(x = 'period', y = 'frequency', label = 'label',size = 0.2), nudge_y=0.3)
                       + scale_size_continuous(guide = None)
                       + guides(color=guide_legend(nrow=40))
                       + theme_minimal()
                       + theme(axis_title_x=element_text(colour = "black",size = 7),
                               axis_title_y=element_text(colour="black", size = 7),
                               axis_title=element_text(face="bold"),
                               axis_text_x=element_text(colour = "black", rotation=45, hjust=1, size = 7),
                               legend_text = element_text(size=7),
                               legend_title = element_text(size=10),
                               legend_key_height = 0.5,
                               legend_key_width = 0.5,
                               figure_size=(5 + max_words, (1 + (max_words * 1.0))),
                               panel_grid_major = element_blank(),
                               panel_background = element_rect(colour = "white", fill = "white"),
                               plot_background = element_rect(colour = 'white',fill = "white"))
                       ).draw(show=False, return_ggplot=True)

                plt.gcf().set_dpi(800)
                picture = plt.show()


            else:
                sys.exit('Incorrect value for plot parameter. Allowed: "heatmap", "line"')

        else:
            sys.exit('Incorrect value for time_freq parameter. Allowed: "ungroup", "Y", "M"')

    elif ngram == 3:
        if time_freq == 'ungroup':
            if plot == 'wordcloud':
                trigram_df = result[['trigram','trigram_freq']]
                trigram_df = trigram_df.dropna()
                trigram_df['trigram'] = trigram_df['trigram'].str.replace(",", " ")
                trigram_df['trigram_freq'] = trigram_df['trigram_freq'].astype(int)

                fig = figure()
                trigram_dict = dict(trigram_df.values)
                wordcloud_trigram = WordCloud(background_color="white",
                                              colormap='tab20b',
                                              width=2000,
                                              height=1200,
                                              margin=10).generate_from_frequencies(trigram_dict)
                plt.figure(dpi=1200)
                plt.imshow(wordcloud_trigram,interpolation="bilinear")
                fig.set_size_inches(10, 6, forward=True)
                plt.axis('off')
                picture = plt.show()

            else:
                sys.exit('Incorrect value for plot parameter. Allowed: "wordcloud"')

        else:
            sys.exit('Incorrect value for time_freq parameter. Allowed: "ungroup"')

    else:
        sys.exit("Incorrect value for ngram parameter. Allowed: 1,2,3.")

    return picture
#%%



# -*- coding: utf-8 -*


import setuptools

with open("README.md", "r") as fh:
    description = fh.read()

    setuptools.setup(
        name="arabica",
        version="1.8.0",
        author="Petr Koráb",
        author_email="xpetrkorab@gmail.com",
        packages=["arabica"],
        description="Python package for text mining of time-series data",
        long_description=description,
        long_description_content_type="text/markdown",
        url="https://github.com/PetrKorab/Arabica",
        python_requires='>=3.8, <3.11',
        install_requires = ['pandas == 1.4.0',
                            'numpy == 1.23.0',
                            'nltk == 3.6.2',
                            'regex == 2022.10.31',
                            'finvader == 1.0.4',
                            'matplotlib == 3.6.0',
                            'matplotlib-inline == 0.1.6',
                            'plotnine == 0.10.1',
                            'wordcloud == 1.9.3',
                            'jenkspy == 0.3.2',
                            'vaderSentiment == 3.3.2',
                            'cleantext == 1.1.4',
                            'mizani == 0.9.2',
                            'pillow == 9.4.0',
                            'ftfy == 6.2.0',
                            'chardet == 5.2.0'],
        license='OSI Approved :: Apache Software License'
    )